@@ -40,6 +40,7 @@
 #include "gui_main.h"
 #include "gtkpixcomm.h"
 
+
 static void	gtk_pixcomm_class_init (GtkPixcommClass *klass);
 static void	gtk_pixcomm_init       (GtkPixcomm *pixcomm);
 static gboolean gtk_pixcomm_expose     (GtkWidget *widget, GdkEventExpose *ev);
@@ -50,6 +51,7 @@ static void build_insensitive_pixbuf (GtkPixcomm *pixcomm);
 
 static GtkMiscClass *parent_class;
 
+
 enum op_t {
   OP_FILL,
   OP_COPY,
@@ -67,6 +69,7 @@ struct op {
   gint x, y;
 };
 
+
 GType
 gtk_pixcomm_get_type(void)
 {
@@ -75,13 +78,13 @@ gtk_pixcomm_get_type(void)
   if (!pixcomm_type) {
     static const GTypeInfo pixcomm_info = {
       sizeof(GtkPixcommClass),
-      NULL,
-      NULL,
+      NULL,		/* base_init */
+      NULL,		/* base_finalize */
       (GClassInitFunc) gtk_pixcomm_class_init,
-      NULL,
-      NULL,
+      NULL,		/* class_finalize */
+      NULL,		/* class_data */
       sizeof(GtkPixcomm),
-      0,
+      0,		/* n_preallocs */
       (GInstanceInitFunc) gtk_pixcomm_init
     };
 
@@ -95,14 +98,10 @@ gtk_pixcomm_get_type(void)
 static void
 gtk_pixcomm_class_init(GtkPixcommClass *klass)
 {
-  GObjectClass *gobject_class;
-  GtkObjectClass *object_class;
-  GtkWidgetClass *widget_class;
+  GtkObjectClass *object_class = GTK_OBJECT_CLASS(klass);
+  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS(klass);
 
   parent_class = g_type_class_peek_parent(klass);
-  gobject_class = G_OBJECT_CLASS(klass);
-  object_class = GTK_OBJECT_CLASS(klass);
-  widget_class = GTK_WIDGET_CLASS(klass);
 
   object_class->destroy = gtk_pixcomm_destroy;
   widget_class->expose_event = gtk_pixcomm_expose;
@@ -208,10 +207,11 @@ gtk_pixcomm_expose(GtkWidget *widget, GdkEventExpose *ev)
   g_return_val_if_fail(GTK_IS_PIXCOMM(widget), FALSE);
   g_return_val_if_fail(ev!=NULL, FALSE);
 
-  if (GTK_WIDGET_VISIBLE(widget) && GTK_WIDGET_MAPPED(widget)) {
+  if (GTK_WIDGET_DRAWABLE(widget)) {
     GtkPixcomm *p;
     GtkMisc *misc;
     gint x, y;
+    gfloat xalign;
     guint i;
 
     p = GTK_PIXCOMM(widget);
@@ -220,12 +220,20 @@ gtk_pixcomm_expose(GtkWidget *widget, GdkEventExpose *ev)
     if (p->actions->len <= 0)
       return FALSE;
 
-    x=(widget->allocation.x * (1.0 - misc->xalign) +
-      (widget->allocation.x + widget->allocation.width
-      - (widget->requisition.width  - misc->xpad * 2)) * misc->xalign) + 0.5;
-    y=(widget->allocation.y * (1.0 - misc->yalign) +
-      (widget->allocation.y + widget->allocation.height
-      - (widget->requisition.height - misc->ypad * 2)) * misc->yalign) + 0.5;
+    if (gtk_widget_get_direction(widget) == GTK_TEXT_DIR_LTR) {
+      xalign = misc->xalign;
+    } else {
+      xalign = 1.0 - misc->xalign;
+    }
+
+    x = floor(widget->allocation.x + misc->xpad
+	+ ((widget->allocation.width - widget->requisition.width) *
+	  xalign)
+	+ 0.5);
+    y = floor(widget->allocation.y + misc->ypad 
+	+ ((widget->allocation.height - widget->requisition.height) *
+	  misc->yalign)
+	+ 0.5);
 
     /* draw! */
     for (i = 0; i < p->actions->len; i++) {
@@ -239,21 +247,23 @@ gtk_pixcomm_expose(GtkWidget *widget, GdkEventExpose *ev)
 
       case OP_COPY:
         if (rop->src->has_mask) {
-          gdk_gc_set_clip_origin(civ_gc, x + rop->x, y + rop->y);
           gdk_gc_set_clip_mask(civ_gc, rop->src->mask);
+          gdk_gc_set_clip_origin(civ_gc, x + rop->x, y + rop->y);
 
-          gdk_draw_drawable(widget->window, civ_gc, rop->src->pixmap,
-                            0, 0,
-			    x + rop->x, y + rop->y,
-			    rop->src->width, rop->src->height);
+          gdk_draw_drawable(widget->window, civ_gc,
+	      rop->src->pixmap,
+	      0, 0,
+	      x + rop->x, y + rop->y,
+	      rop->src->width, rop->src->height);
 
-          gdk_gc_set_clip_mask(civ_gc, NULL);
           gdk_gc_set_clip_origin(civ_gc, 0, 0);
+          gdk_gc_set_clip_mask(civ_gc, NULL);
         } else {
-          gdk_draw_drawable(widget->window, civ_gc, rop->src->pixmap,
-                          0, 0,
-			  x + rop->x, y + rop->y,
-			  rop->src->width, rop->src->height);
+          gdk_draw_drawable(widget->window, civ_gc,
+	      rop->src->pixmap,
+	      0, 0,
+	      x + rop->x, y + rop->y,
+	      rop->src->width, rop->src->height);
 	}
         break;
 
