@@ -1914,7 +1914,7 @@ static void mapgenerator4(void)
   /* no islands with mass >> sqr(min(xsize,ysize)) */
 
   if (game.nplayers < 2 || map.landpercent > 80) {
-    map.generator = 3;
+    map.startpos = 1;
     return;
   }
 
